﻿#include "dma.h"

unsigned int sys_kernel_va_to_pa(unsigned int va) {
  if (va >= 0xE0000000)
    return va;
  if (va >= 0xa0000000)
    return (va - 0xa0000000);
  if (va < 0x80000000)
    return (va + 0x40000000);
  return sys_kernel_va_to_pa(va | 0x20000000); // cache address
}

void DMA_Configure_chain(
    ChainDmaPort *_current, //Указатель на структуру текущего звена
    ChainDmaPort *_next, //Указатель на структуру следующего звена
    unsigned int _run,   //Бит RUN, запуск звена
    unsigned int _im,    //Бит IM, разрещающий прерывание по завершению звена
    unsigned int _chen,  //Бит CHEN, разрешение дальнейшей самоинициализации
    unsigned int *_data, //Адрес данных для передачи
    unsigned int _size)  //Размер массива данных
{
  unsigned int csr_data;
  unsigned int size_cor;

  if ((_size & 0x7) != 0)
    size_cor = _size + 0x8; //Выравнивание данных
  else
    size_cor = _size;

  csr_data = (((size_cor / 8) - 1) << 16) | (_chen << 12) | (_im << 13) |
             (0xf << 2) | (_run);

  _current->_CSR = csr_data;
  _current->_CP = sys_kernel_va_to_pa((unsigned int)&_next->_Reserved) | 1;
  _current->_IR = sys_kernel_va_to_pa((unsigned int)_data);
    _current->_Reserved = 0;

}


void DMA_Load_chain(ChainDmaPort *_first )        //Указатель на первое звено цепочки
{
    IR_VPOUT_CH  = _first->_IR;
    CP_VPOUT_CH  = _first->_CP;
    CSR_VPOUT_CH = _first->_CSR;
}
